@@ -444,8 +444,8 @@ macdefine(mac, vclass, id, value)
 		sm_dprintf(")\n");
 	}
 #if USE_EAI && 0
-	if (('j' == id || 'm' == id) && !addr_is_ascii(value))
-		return an error/warning to caller and let them handle it.
+//	if (('j' == id || 'm' == id) && !addr_is_ascii(value))
+//		return an error/warning to caller and let them handle it.
 #endif
 
 	if (mac->mac_rpool == NULL)
@@ -765,7 +765,7 @@ wordinclass(str, cl)
 	STAB *s;
 #if _FFR_DYN_CLASS
 	MAP *map;
-	int status;
+	int stat;
 	char *p;
 	char key[MAXLINE];
 
@@ -794,16 +794,16 @@ wordinclass(str, cl)
 		sm_strlcat(key, ":", sizeof(key));
 	}
 	sm_strlcat(key, str, sizeof(key));
-	status = EX_OK;
-	p = (map->map_class->map_lookup)(map, key, NULL, &status);
+	stat = EX_OK;
+	p = (map->map_class->map_lookup)(map, key, NULL, &stat);
 	if (NULL != p)
 		return true;
-	if ((EX_OK == status && NULL == p) || EX_NOTFOUND == status)
+	if ((EX_OK == stat && NULL == p) || EX_NOTFOUND == stat)
 		return false;
 
 	sm_syslog(LOG_WARNING, CurEnv->e_id,
-		"dynamic class: A{%s}: map lookup failed: key=%s, status=%d",
-		map->map_mname, key, status);
+		"dynamic class: A{%s}: map lookup failed: key=%s, stat=%d",
+		map->map_mname, key, stat);
 
 	/* Note: this error is shown to the client, so do not "leak" info */
 	usrerr("451 4.3.1 temporary error");
