@@ -345,13 +345,15 @@ mci_clear(mci)
 	if (bitset(MCIF_TLSACT, mci->mci_flags) && mci->mci_ssl != NULL)
 		SM_SSL_FREE(mci->mci_ssl);
 #endif
+#if _FFR_SE_TA_ID
+	mci->mci_c_se = NULL;
+#endif
 
 	/* which flags to preserve? */
 	mci->mci_flags &= MCIF_CACHED;
 	mactabclear(&mci->mci_macro);
 }
 
-
 /*
 **  MCI_GET -- get information about a particular host
 **
@@ -379,7 +381,10 @@ mci_get(host, m)
 	(void) mci_scan(NULL);
 
 	if (m->m_mno < 0)
+	{
 		syserr("!negative mno %d (%s)", m->m_mno, m->m_name);
+		/* NOTREACHED */
+	}
 
 	s = stab(host, ST_MCI + m->m_mno, ST_ENTER);
 	mci = &s->s_mci;
