@@ -307,8 +307,8 @@ invalidaddr(addr, delimptr, isrcpt)
 			char saved = *addr;
 
 			*addr = '\0';
-			usrerr("553 5.1.0 Address \"%s\" too long (%d bytes max)",
-			       b, MAXNAME - 1);	/* EAI:ok */
+			usrerr("553 5.1.0 Address too long (%d bytes max)",
+			       MAXNAME - 1);	/* EAI:ok */
 			*addr = saved;
 			result = true;
 			goto delim;
@@ -375,8 +375,8 @@ hasctrlchar(addr, isrcpt, complain)
 			if (complain)
 			{
 				(void) shorten_rfc822_string(b, MAXNAME - 1);	/* EAI:ok */
-				usrerr("553 5.1.0 Address \"%s\" too long (%d bytes max)",
-				       b, MAXNAME - 1);	/* EAI:ok */
+				usrerr("553 5.1.0 Address too long (%d bytes max)",
+				       MAXNAME - 1);	/* EAI:ok */
 				return true;
 			}
 			result = "too long";
@@ -399,7 +399,7 @@ hasctrlchar(addr, isrcpt, complain)
 				break;
 			}
 		}
-		if (!SMTPUTF8 && !EightBitAddrOK && (*addr & 0340) == 0200)
+		if (!SMTP_UTF8 && !EightBitAddrOK && (*addr & 0340) == 0200)
 		{
 			setstat(EX_USAGE);
 			result = "8-bit character";
