@@ -44,7 +44,7 @@ static char	SmtpReplyBuffer[MAXLINE];	/* buffer for replies */
 static bool	SmtpNeedIntro;		/* need "while talking" in transcript */
 
 /*
-**  SMTPCCLRSE -- clear session related data in envelope
+**  SMTPCLRSE -- clear session related data in envelope
 **
 **	Parameters:
 **		e -- the envelope.
@@ -162,6 +162,26 @@ smtpinit(m, mci, e, onlyhelo)
 
 	mci->mci_state = MCIS_OPENING;
 	clrsessenvelope(e);
+#if _FFR_SE_TA_ID
+	if (NULL == mci->mci_c_se)
+	{
+		char c_se_id[MAXQFNAME + 4];
+
+		(void) sm_snprintf(c_se_id, sizeof(c_se_id),
+			"CS%s%X", e->e_id + 2, e->e_c_se_cnt);
+		mci->mci_c_se = sm_rpool_strdup_x(mci->mci_rpool, c_se_id);
+		++e->e_c_se_cnt;	/* always? */
+		if (tTd(94, 8))
+			sm_dprintf("smtpinit: set: mci_c_se=%s, qid=%s\n",
+				mci->mci_c_se, e->e_id);
+	}
+	SM_ASSERT(NULL != mci->mci_c_se);
+	SM_ASSERT(strlen(mci->mci_c_se) < MAXQFNAME + 4);
+	e->e_c_se = sm_rpool_strdup_x(e->e_rpool, mci->mci_c_se);
+	if (tTd(94, 8))
+		sm_dprintf("smtpinit: qid=%s, " C_SE_ID "=%s, ct_se_cnt=%u\n",
+			e->e_id, e->e_c_se, e->e_c_se_cnt);
+#endif
 
 	/*
 	**  Get the greeting message.
@@ -1715,6 +1735,7 @@ attemptauth(m, mci, e, sai)
 			break;
 #   endif
 		  default:
+			addrsize = 0;	/* just to shut up the compiler */
 			break;
 		}
 		if (iptostring(&CurHostAddr, addrsize,
@@ -2094,7 +2115,7 @@ smtpmailfrom(m, mci, e)
 			e->e_smtputf8 = true;
 	}
 
-	if (e->e_smtputf8 && !SMTPUTF8)
+	if (e->e_smtputf8 && !SMTP_UTF8)
 	{
 		extern char MsgBuf[];
 
@@ -2189,6 +2210,9 @@ smtpmailfrom(m, mci, e)
 	if (bitset(MCIF_DSN, mci->mci_flags))
 	{
 		if (e->e_envid != NULL &&
+#if defined(D_NOENVID)
+		    !iscltflgset(e, D_NOENVID) &&
+#endif
 		    SPACELEFT(optbuf, bufp) > strlen(e->e_envid) + 7)
 		{
 			(void) sm_snprintf(bufp, SPACELEFT(optbuf, bufp),
@@ -2294,6 +2318,28 @@ smtpmailfrom(m, mci, e)
 		smtpmessage("MAIL From:<@%s%c%s>%s", m, mci, MyHostName,
 			    *bufp == '@' ? ',' : ':', bufp, optbuf);
 	}
+#if _FFR_SE_TA_ID
+	do {
+		char c_ta_id[MAXQFNAME + 4];
+		(void) sm_snprintf(c_ta_id, sizeof(c_ta_id),
+			"CT%s%X", e->e_id + 2, e->e_c_ta_cnt);
+		e->e_c_ta = sm_rpool_strdup_x(e->e_rpool, c_ta_id);
+		if (tTd(94, 8))
+			sm_dprintf("smtpmail: set: e_c_ta=%s, qid=%s\n",
+				e->e_c_ta, e->e_id);
+	} while (0);
+	++e->e_c_ta_cnt;	/* always? */
+	if (tTd(94, 8))
+		sm_dprintf("smtpmail: qid=%s, " C_TA_ID "=%s, ct_ta_cnt=%u\n",
+			e->e_id, e->e_c_ta, e->e_c_ta_cnt);
+
+	/* fail is mci_c_se is not set? */
+	if (NULL == e->e_c_se && NULL != mci->mci_c_se)
+		e->e_c_se = sm_rpool_strdup_x(e->e_rpool, mci->mci_c_se);
+	if (tTd(94, 8))
+		sm_dprintf("smtpmail: qid=%s, " C_SE_ID "=%s, ct_se_cnt=%u\n",
+			e->e_id, e->e_c_se, e->e_c_se_cnt);
+#endif
 	SmtpPhase = mci->mci_phase = "client MAIL";
 	sm_setproctitle(true, e, "%s %s: %s", qid_printname(e),
 			CurHostName, mci->mci_phase);
@@ -2661,6 +2707,7 @@ smtpdata(m, mci, e, ctladdr, xstart)
 	int xstat;
 	int timeout;
 	char *enhsc;
+	char *where, *hdrname;
 
 	/*
 	**  Check if connection is gone, if so
@@ -2675,6 +2722,8 @@ smtpdata(m, mci, e, ctladdr, xstart)
 	}
 
 	enhsc = NULL;
+	where = NULL;
+	hdrname = NULL;
 
 	/*
 	**  Send the data.
@@ -2786,8 +2835,11 @@ smtpdata(m, mci, e, ctladdr, xstart)
 		**  Output the actual message.
 		*/
 
-		if (!(*e->e_puthdr)(mci, e->e_header, e, M87F_OUTER))
+		if (!(*e->e_puthdr)(mci, e->e_header, e, M87F_OUTER, &hdrname))
+		{
+			where = "header";
 			goto writeerr;
+		}
 
 		if (tTd(18, 101))
 		{
@@ -2796,7 +2848,10 @@ smtpdata(m, mci, e, ctladdr, xstart)
 		}
 
 		if (!(*e->e_putbody)(mci, e, NULL))
+		{
+			where = "body";
 			goto writeerr;
+		}
 
 		/*
 		**  Cleanup after sending message.
@@ -2840,7 +2895,10 @@ smtpdata(m, mci, e, ctladdr, xstart)
 	if (sm_io_fprintf(mci->mci_out, SM_TIME_DEFAULT, "%s.%s",
 			bitset(MCIF_INLONGLINE, mci->mci_flags) ? m->m_eol : "",
 			m->m_eol) == SM_IO_EOF)
+	{
+		where = "EOM";
 		goto writeerr;
+	}
 	if (TrafficLogFile != NULL)
 		(void) sm_io_fprintf(TrafficLogFile, SM_TIME_DEFAULT,
 				     "%05d >>> .\n", (int) CurrentPid);
@@ -2910,7 +2968,11 @@ smtpdata(m, mci, e, ctladdr, xstart)
 		(void) bfrewind(e->e_dfp);
 
 	errno = mci->mci_errno;
-	syserr("+451 4.4.1 timeout writing message to %s", CurHostName);
+	syserr("+451 4.4.2 timeout writing message to %s, where=%s%s%s",
+		CurHostName, where,
+		NULL == hdrname ? "" : ", header=",
+		NULL == hdrname ? "" : hdrname
+		);
 	smtpquit(m, mci, e);
 	return EX_TEMPFAIL;
 }
